from aheadworks_bitbucket_manager.api.bitbucket_api_manager import BitbucketApiManager
from aheadworks_bitbucket_manager.model.data.bitbucket import BitbucketConfig
import json
import subprocess
import os
import shutil
import json


class PipelineManager:

    def send_pipelines_to_workspace(self, bitbucket_workspace):
        # Install and configure git
        os.system('apk add git')
        os.system('git config --global user.email "bot@raveinfosys.com"')
        os.system('git config --global user.name "Automatic update"')

        bitbucket_api_manager = BitbucketApiManager(bitbucket_config=BitbucketConfig(bitbucket_workspace=bitbucket_workspace))
        # we only update modules that has 'module-' in the name
        full_repo_list = bitbucket_api_manager.get_repositories(name_contains = "module-")
        print(full_repo_list)

        new_bitbucket_pipelines = self.get_new_pipelines()

        os.system('mkdir tmpgit')
        os.chdir("./tmpgit")
        for url in full_repo_list:
            # skip the source repo module-boilerplate
            if url.find('module-boilerplate') == -1:
                self.update_repository_pipelines(url, new_bitbucket_pipelines)
        os.chdir("..")
        os.system('rm -r tmpgit')

    def get_new_pipelines(self):
        readFile = open("bitbucket-pipelines.yml")
        lines = readFile.readlines()
        readFile.close()
        # The Last line contains a weird comment with the date - strip it
        if lines[-1][0:1] == '#':
            del lines[-1]
        # Strip the "deploy pipelines" pipeline from the result so it won't be possible to send new pipelines from any M2 repo
        firstIndex = None
        lastIndex = None
        firstInstructionIndex = None
        lastInstructionIndex = None
        for index, line in enumerate(lines):
            if line.find('&deployPipelines') != -1:
                firstInstructionIndex = index
            # TODO: This line is going to change once we migrate to deploy-tools library
            elif line.find("send-pipelines-to-workspace") != -1:
                lastInstructionIndex = index + 1
            elif line.find('deploy-pipeline') != -1:
                firstIndex = index
            elif line.find('*deployPipelines') != -1:
                lastIndex = index + 1
        del lines[firstIndex:lastIndex]
        del lines[firstInstructionIndex:lastInstructionIndex]
        return "".join(lines)

    def update_repository_pipelines(self, url, new_bitbucket_pipelines):
        subprocess.Popen(['git', 'clone', url]).communicate()
        module = url.split('/')[-1].replace('.git', '')
        os.chdir(module)
        subprocess.Popen(['ls', '-la']).communicate()
        try:
            output = subprocess.check_output("git branch -al --no-merged", shell=True, text=True)
        except subprocess.CalledProcessError as e:
            # is it a broken/new repo ???
            print(e.returncode)
            print(e.output)
            return

        branches = output.splitlines()
        # 'splitlines' -- from str to list
        # branches_to_update list for update release or develop branches only
        branches_to_update = []

        # always add develop branch
        is_dev_branch_exists = subprocess.check_output("git branch -lr 'origin/develop'", shell=True)
        if is_dev_branch_exists:
            branches_to_update.append("develop")

        # add all unmerged 'release/' branches
        for branch in branches:
            clean_branch = branch.strip().removeprefix("remotes/origin/")
            if clean_branch.startswith('release'):
                branches_to_update.append(clean_branch)
            elif clean_branch.startswith('bugfix'):
                branches_to_update.append(clean_branch)
            elif clean_branch.startswith('feature'):
                branches_to_update.append(clean_branch)

        for branch in branches_to_update:
            subprocess.Popen(['git', 'checkout', branch]).communicate()
            f = open("bitbucket-pipelines.yml", "w")
            f.write(new_bitbucket_pipelines)
            f.close()
            subprocess.Popen(['git', 'add', "bitbucket-pipelines.yml"]).communicate()
            subprocess.Popen(['git', 'commit', '-m' "[SKIP CI] update bitbucket-pipelines.yml"]).communicate()
        subprocess.Popen(['git', 'push', '--all', 'origin']).communicate()
        os.chdir("..")




from aheadworks_bitbucket_manager.model.ssh_manager import SshManager


class DbManager:

    def __init__(self):
        self.ssh_manager = SshManager()

    def create_dump(self, path_to_sh, connection):
        command = 'test -f ' + path_to_sh + ' && echo True || echo False'
        file_exists = self.ssh_manager.run_ssh_command(command, connection)
        file_exists = eval(file_exists[0].strip()) if file_exists else False
        if file_exists:
            result = 'Db dump created'
            self.ssh_manager.run_ssh_command(path_to_sh, connection)
        else:
            result = 'Db dump missed'

        return result



from aheadworks_bitbucket_manager.api.bitbucket_api_manager import BitbucketApiManager
from aheadworks_bitbucket_manager.model.data.bitbucket import BitbucketConfig
from aheadworks_core.api.composer_manager import ComposerManager
from aheadworks_core.api.teamwork_manager import TeamworkManager
from aheadworks_core.api.discord_api_manager import DiscordApiManager
from aheadworks_core.api.magento_manager import MagentoManager
from aheadworks_core.api.file_manager import FileManager
from aheadworks_core.model.parser.json import Json as JsonParser
from datetime import datetime
import boto3
import copy
import json
import os
import re
import shutil
import subprocess
import requests

class ReleaseManager:
    """api manager for release"""

    RELEASE_PACK_TASK_LABEL = 'RELEASE-PACK'
    PD_TASK_LABEL = 'PD'
    TEST_TASK_LABEL = 'TEST'

    def __init__(self, api_config):
        self.teamwork_manager = TeamworkManager(api_config)
        self.discord_api_manager = DiscordApiManager()
        self.magento_manager = MagentoManager()
        self.file_manager = FileManager()
        self.json_parser = JsonParser()
        self.aws_s3 = boto3.resource('s3')

    def teamwork_release(self, project_id, composer_file, discord_bot_url, path_to_files, assign_to):
        module_version = self.json_parser.get_variable_from_file('version', composer_file)

        print(f"project id: {project_id}")
        print(f"module version: '{module_version}'")
        print(f"discord bot url: {discord_bot_url}")
        print(f"path to files: {path_to_files}")
        print(f"assign to, account id: {assign_to}")

        if not project_id:
            print('project_id is empty, skip teamwork release.')
            return False

        links = self.teamwork_manager.find_tasks_by_tags([f"{project_id}-{module_version}"])
        release_list = self.teamwork_manager.find_tasks_by_tags([f"{project_id}-{module_version}", self.RELEASE_PACK_TASK_LABEL])
        pd_list = self.teamwork_manager.find_tasks_by_tags([f"{project_id}-{module_version}", self.PD_TASK_LABEL])
        test_list = self.teamwork_manager.find_tasks_by_tags([f"{project_id}-{module_version}", self.TEST_TASK_LABEL])
        release_tasks_count = len(release_list) + len(pd_list) + len(test_list)
        if release_tasks_count != 3:
            print(f"release: {release_list}")
            print(f"test: {test_list}")
            print(f"pd: {pd_list}")
            raise Exception(f'There should be exactly 3 release tasks, got {release_tasks_count}.')

        release_task_id = release_list[0]['id']
        test_task_id = test_list[0]['id']
        pd_task_id = pd_list[0]['id']

        tasklist_id = release_list[0]['tasklistId']
        tasklist = self.teamwork_manager.get_tasklist(tasklist_id)
        project_id = str(tasklist['tasklist']['projectId'])
        project_name = tasklist['included']['projects'][project_id]['name']

        # add attachments to the task RELEASE-PACK
        os.system(f"ls -al {path_to_files}")
        file_names = [os.path.join(path_to_files, file) for file in os.listdir(path_to_files)]
        self.teamwork_manager.add_attachments_to_task(release_task_id, file_names)

        # assign release pack to user
        if (not assign_to == "TEAMWORK_ACCOUNT_ID"):
            self.teamwork_manager.reassign(release_task_id, assign_to)

        module_dependencies = self.magento_manager.get_module_dependencies_from_composer(composer_file)
        composer_package_name = ','.join(list(map(lambda x: x['full_module_name'], module_dependencies.values())))
        self.teamwork_manager.add_comment(release_task_id, f'Composer Package Name:\n{composer_package_name}')

        # Set PD and TEST to Done
        self.teamwork_manager.close_issue(pd_task_id)
        self.teamwork_manager.close_issue(test_task_id)

        release_link = self.teamwork_manager.get_issue_url(release_task_id)
        test_link = self.teamwork_manager.get_issue_url(test_task_id)
        pd_link = self.teamwork_manager.get_issue_url(pd_task_id)

        msg = '{} {}\n'.format(project_name, module_version)
        msg += f'\n{self.RELEASE_PACK_TASK_LABEL}: {release_link}\n{self.TEST_TASK_LABEL}: {test_link}\n{self.PD_TASK_LABEL}: {pd_link}'
        print(msg)

        # ????? msg += '\n' + self.jira_api_manager.get_release_report_all_issues_url(jira_project_key, version.id)
        self.discord_api_manager.send_msg(discord_bot_url, msg)

        return True

    def build_swagger_web_api_doc(
            self,
            path_to_module,
            magento_url,
            magento_path_on_server='/var/www/html'
    ):
        subprocess.check_call("nginx -g \"daemon on;\" & docker-php-entrypoint php-fpm -R &", shell=True)
        aws_bucket_name = 'aheadworks_cdn'
        aws_swagger_web_api_doc_path = 'swagger_web_api_doc/'
        vendor_path = "/var/www/html/vendor/aheadworks/"
        parent_composer = path_to_module + "composer.json"

        # here we define name from /etc/module.xml
        parent_module_name_from_xml = self.magento_manager.get_module_name(path_to_module)
        # in module_list will be added all modules from suggests
        module_list = parent_module_name_from_xml
        if not os.path.isfile(f"{path_to_module}/etc/webapi.xml"):
            return f'Skip Web API doc generation: file etc/webapi.xml has been not found for module {parent_module_name_from_xml}'
        try:
            with open(parent_composer) as f:
                composer = json.load(f)

            parent_module_name = composer['name']
            parent_module_version = composer['version']
            os.chdir(magento_path_on_server)
            os.system(f"composer require {parent_module_name}:{parent_module_version}")
            # fast fix: some modules have no modules in suggests
            if 'suggests' in composer:
                try:
                    for name, version in composer['suggests'].items():
                        os.system(f"composer require {name}:{version}")
                        name_wo_aheadworks = name.split('/')[1]
                        vendor_module_path = vendor_path + name_wo_aheadworks
                        # get names from /etc/module.xml
                        module_name_from_xml = self.magento_manager.get_module_name(vendor_module_path)
                        # add names to list
                        module_list += "," + module_name_from_xml
                except Exception as error:
                    raise Exception(error)
            # generate url for main and suggests modules
            magento_request_url = '/generate_web_api_json.php?module_names={}'.format(module_list)
            swagger_json_page = magento_url + magento_request_url
            # this action need because first call swagger script return error output(maybe modules cant loaded to fast)
            requests.get(swagger_json_page)
            # here we get json output from main and suggests modules
            swagger_json = requests.get(swagger_json_page).text

            try:
                json.loads(swagger_json)
            except Exception as error:
                print(f"Invalid response from Swagger:\n{swagger_json}\n\n")
                raise Exception(error)

            s3_result = self.aws_s3.Bucket(aws_bucket_name).put_object(
                Key=f"{aws_swagger_web_api_doc_path}{parent_module_name_from_xml.lower()}_latest.json",
                Body=swagger_json,
                ACL='public-read'
            )
        except Exception as error:
            raise Exception(error)

        result = f'Web Api Doc Path: https://media.aheadworks.com/{s3_result.key}\n'
        result += f'Magento Request Url: {magento_request_url}\n'
        return result

    def build_ecommerce_pack(self, bitbucket_workspace, bitbucket_repo_slug):
        ComposerManager.init_extra_repos()

        build_dir = os.getcwd()
        relative_path = "/app/code/Aheadworks"
        self.file_manager.create_empty_dir(f"{build_dir}/community{relative_path}")
        self.file_manager.create_empty_dir(f"{build_dir}/enterprise{relative_path}")

        artifacts_dir = "/build_archives"
        self.file_manager.create_empty_dir(artifacts_dir)

        composer = self._load_composer_file(bitbucket_repo_slug, '.')
        core_module_name = composer['name']
        core_module_version = composer['version']
        # check if composer have version that is in composer.json
        print("If you see an error just below, then the composer does not see the version of the package.")
        print("Did you tag latest release?")
        subprocess.run(["composer", "show", "-a", core_module_name, core_module_version], check=True)

        module_dependencies = self.magento_manager.get_module_dependencies('./')
        ComposerManager.require_magento_module(core_module_name, composer['version'])

        # @todo use self.magento_manager.download_modules_from_git(path_to_module, tmp_dir_m2_modules)
        magento_module_info = dict()
        artifacts = list()
        # Preinstall suggested modules with composer into /var/www/html/vendor/aheadworks
        platform_dependencies = self.magento_manager.get_platform_module_dependencies(core_module_name).keys()
        for full_module_name, module_item in module_dependencies.items():
            if self.magento_manager.is_suggested_module(build_dir, full_module_name) or full_module_name in platform_dependencies:
                ComposerManager.require_magento_module(full_module_name)

        # full_module_name          vendor/module-name
        # module_name               module-name
        # magento_package_name      ModuleName
        for full_module_name, module_item in module_dependencies.items():
            module_name = module_item['module_name']
            composer_install_path = f"/var/www/html/vendor/aheadworks/{module_name}"
            magento_package_name = self._get_magento_package_name(full_module_name, composer_install_path)
            magento_module_info[full_module_name] = magento_package_name
            module_composer = self._load_composer_file(full_module_name, composer_install_path)

            print(f"Building {full_module_name} from {composer_install_path} as {module_name}...")
            self._prepare_for_publishing(
                composer_install_path,
                magento_package_name,
                module_composer["version"]
            )
            artifacts.append(
                self._pack_module(composer_install_path, f"{artifacts_dir}/{module_name}.zip")
            )

            # 'community' 'enterprise' or 'any'
            module_plaftorm = module_item.get('platform', 'any')
            for platform in ['community', 'enterprise']:
                if module_plaftorm == 'any' or module_plaftorm == platform:
                    target_module_path = f"{build_dir}/{platform}{relative_path}/{magento_package_name}"
                    shutil.copytree(composer_install_path, target_module_path)

        # Now build store packages
        for platform in ['community', 'enterprise']:
            filename = f"aw_m2_{magento_module_info[core_module_name]}-{composer['version']}.{platform}_edition.zip"
            artifacts.append(
                self._pack_module(f"{build_dir}/{platform}", f"{artifacts_dir}/{filename}", "app")
            )

        self._upload_artifacts(bitbucket_workspace, bitbucket_repo_slug, artifacts)

    # Sample metapackage products:
    # https://bitbucket.org/awm2ext/b2b-suite/
    # https://bitbucket.org/awm2ext/b2b-suite-hyva/
    def build_metapackage_pack(self, bitbucket_workspace, bitbucket_repo_slug, artifacts_dir):
        '''
          for b2b
          'aheadworks/module-ui-components' is NEVER renamed
          1. with meatapackage composer file:
            - replace in the metapackage 'requre' section module names so they have '-b2b-suite' postfix
          2. with each dependent module composer file:
            - add '-b2b-suite' postfix to the name
            - get 'require' section
            - replace in the 'require' and 'suggests' sections only module names that matches the metapackage 'require' section
            - replace in the 'suggests' section '-hyva' with '-b2b-suite-hyva'  (if there are any) 
    
          for b2b-hyva
          'aheadworks/module-ui-components' is NEVER renamed
          1. with meatapackage composer file:
            - replace in the 'requre' section module names so the postfix '-hyva' becomes '-b2b-suite-hyva'
          2. with each dependent module composer file:
            - replace '-hyva' with '-b2b-suite-hyva' in the name
            - get 'require' section
            - replace in the 'require' and 'suggests' sections only module names that matches the metapackage 'require' section
            - replace in the 'suggests' section '-hyva' postfix with '-b2b-suite-hyva'  (if there are any) 

        '''

        upload_env = f"REPO_URL={os.getenv('AW_COMPOSER_API_URL')} REPO_LOGIN={os.getenv('AW_COMPOSER_API_LOGIN')} REPO_TOKEN={os.getenv('AW_COMPOSER_API_PASSWORD')}"

        # prepare FS
        build_dir = os.getcwd()
        if not artifacts_dir:
            artifacts_dir = "/build_archives"
        self.file_manager.create_empty_dir(artifacts_dir)

        path = f"{build_dir}/app/code/Aheadworks"
        self.file_manager.create_empty_dir(path)

        # Get dependent packages list
        composer = self._load_composer_file(bitbucket_repo_slug, '.')

        # Install dependent packages via Composer first
        for full_module_name in composer["require"]:
            ComposerManager.require_magento_module(full_module_name)

        artifacts = []
        for full_module_name in composer["require"]:
            # full_module_name     - 'aheadworks/module-name'
            # module_name          - 'module-name'
            # magento_package_name - 'ModuleName'
            module_name = full_module_name.split('/')[1]
            composer_install_path = f"/var/www/html/vendor/aheadworks/{module_name}"
            module_composer = self._load_composer_file(full_module_name, composer_install_path)
            magento_package_name = self._get_magento_package_name(full_module_name, composer_install_path)
            module_path = f"{path}/{magento_package_name}"

            print(f"Building {full_module_name} as {module_name} in {module_path}...")
            self._prepare_for_publishing(
                composer_install_path,
                magento_package_name,
                module_composer['version']
            )
            shutil.copytree(composer_install_path, module_path)

            # changing modules name in module_folder in parent directory(build directory)
            old_name = module_composer['name']
            if not self._is_conflicted_module(old_name):
                # strip -hyva first and then add postfix
                module_name = self._strip_end(module_name, "-hyva")
                metapackage_module_name = f"{module_name}-{bitbucket_repo_slug}"

                new_name = self._strip_end(module_composer['name'], "-hyva")
                new_name = f"{new_name}-{bitbucket_repo_slug}"

                print(f"\t\u21E8 Renaming {old_name} to {new_name}")
                module_composer['name'] = new_name
            else:
                metapackage_module_name = module_name

            with open(f"{module_path}/composer.json", 'w') as f:
                json.dump(module_composer, f, indent=4)

            # changing module names in the dependent 'require' and 'suggest' section
            modules_to_replace = copy.deepcopy(composer["require"])
            replacements = {}
            for m in modules_to_replace:
                if self._is_conflicted_module(m):
                    continue
                # strip "-hyva" if any
                new_name = self._strip_end(m, "-hyva")
                replacements[m] = f"{new_name}-{bitbucket_repo_slug}"

            print(f"\t\u21E8 Replacing requires with {replacements}")
            self._replace_dependencies(f"{module_path}/composer.json", replacements)

            # replacing '-hyva' with '-b2b-suite-hyva' in the dependent 'suggests' section
            replacements = {}
            if "suggests" in module_composer:
                for s in module_composer["suggests"]:
                    if self._is_conflicted_module(s):
                        continue
                    if s.startswith("aheadworks/") and s.endswith('-hyva') and not s.endswith(bitbucket_repo_slug):
                        # strip "-hyva" if any
                        new_name = self._strip_end(s, "-hyva")
                        replacements[s] = f"{new_name}-{bitbucket_repo_slug}"
                        print(f"\t\u21E8 Replacing {s} with {replacements[s]}")

            print(f"\t\u21E8 Replacing suggests with {replacements}")
            self._replace_dependencies(f"{module_path}/composer.json", replacements)
            
            zip_package_path = f"{artifacts_dir}/{metapackage_module_name}.zip"
            artifacts.append(
                self._pack_module(module_path, zip_package_path)
            )
            upload_as = f"{metapackage_module_name}-{module_composer['version']}.zip"
            os.system(
                f"echo '{upload_env} python3 -m aheadworks_composer_manager send-package --filename={upload_as} {zip_package_path}' >> {artifacts_dir}/upload")
            # ??? Why we upload to Composer server in this noobish manner instead of using API ???
            composer_server_base_path = f"{os.getenv('AW_COMPOSER_PACKAGES_ROOT')}/aheadworks"
            os.system(
                f"ssh -T {os.getenv('AW_COMPOSER_SSH_URL')} 'mkdir -p {composer_server_base_path}/{metapackage_module_name}'")
            os.system(
                f"scp {zip_package_path} {os.getenv('AW_COMPOSER_SSH_URL')}:{composer_server_base_path}/{metapackage_module_name}/{upload_as}")

        # Here we update parent composer.json
        data = self._load_composer_file(bitbucket_repo_slug, '.')
        core_version = composer['version']

        # Update module names in the parent composer.json 'require' section
        # in dict, we're adding name and version from require and then update with it composer.json
        updated_require = {}
        for key, value in data['require'].items():
            if 'aheadworks/module' in key and not self._is_conflicted_module(key):
                updated_key = self._strip_end(key, '-hyva') + bitbucket_repo_slug
                updated_require[updated_key] = value
                print(f"\t\u21E8 Updating {updated_key} in parent composer.json require with {value}")
            else:
                updated_require[key] = value

        data['require'] = updated_require

        with open('composer.json', 'w') as f:
            json.dump(data, f, indent=4)

        # build package
        artifacts.append(
            self._pack_module(".", f"{artifacts_dir}/{bitbucket_repo_slug}.zip", "composer.json")
        )

        # store package
        for platform in ['community', 'enterprise']:
            filename = f"aw_m2_{bitbucket_repo_slug}-{core_version}.{platform}_edition.zip"
            artifacts.append(
                self._pack_module(build_dir, f"{artifacts_dir}/{filename}", "app")
            )

        self._upload_artifacts(bitbucket_workspace, bitbucket_repo_slug, artifacts)

    def build_mm_pack(self):
        # basically marketplace pack is an ecommerce pack where all modules have -subscription postfix in their names,
        # so we assume that the ecommerce pack has been built so far
        prebuilt_packages_dir = "/build_archives"
        working_dir = "/tmp/unpack"
        package_dir = "/tmp/packages"
        self.file_manager.create_empty_dir(working_dir)
        self.file_manager.create_empty_dir(package_dir)
        os.system(f"ls -l {prebuilt_packages_dir}")
        # Extract all packages first
        for filename in os.listdir(prebuilt_packages_dir):
            package_fullpath = os.path.join(prebuilt_packages_dir, filename)
            # TODO: process *.zip only
            if os.path.isfile(package_fullpath):
                sources_dir = os.path.join(working_dir, filename)
                print(package_fullpath)
                os.system(f"unzip -q {package_fullpath} -d {sources_dir}")

        # Now gather module names
        replacements = {}
        composer_files = self.file_manager.find_all('composer.json', working_dir)
        print(composer_files) # debug
        for composer_file in composer_files:
            package_name = self.json_parser.get_variable_from_file('name', composer_file)
            if not package_name in replacements and not self._is_conflicted_module(package_name):
                replacements[package_name] = f"{package_name}-subscription"

        print(replacements) # debug
        # Replace all found modules names from "vendor/module_name" to "vendor/module_name-subscription"
        # across all composer.json (this includes requires/suggests sections and possibly even more)
        for composer_file in composer_files:
            self._replace_dependencies(composer_file, replacements)

        # Create '-subscription' packages
        for originalname in os.listdir(working_dir):
            sources_fullpath = os.path.join(working_dir, originalname)

            split_name, split_extension = os.path.splitext(originalname)
            target_filename = f"{split_name}-subscription{split_extension}"
            target_fullpath = os.path.join(package_dir, target_filename)
            print(target_fullpath) # debug
            self._pack_module(sources_fullpath, target_fullpath)

        # Copy newly built -subscription packages to /build_archives
        os.system(f"cp {package_dir}/* {prebuilt_packages_dir}")

    def _is_conflicted_module(self, name):
        return name == "aheadworks/module-ui-components"

    def _replace_dependencies(self, path_to_composer, replacements):
        fin = open(path_to_composer, "rt")
        data = fin.read()
        fin.close()
        for module_name in replacements:
            # we intentionally quote vendor/package_name string to be "vendor/package_name".
            # otherwise vendor/package_name_subname could be renamed
            # into vendor/package_name-subscription_subname
            # instead of vendor/package_name_subname-subscription
            # Service module module-ui-components should be filtered out at that point
            data = data.replace(f'"{module_name}"', f'"{replacements[module_name]}"')

        fin = open(path_to_composer, "wt")
        fin.write(data)
        fin.close()

    def _strip_end(self, text, suffix):
        if suffix and text.endswith(suffix):
            return text[:-len(suffix)]
        return text

    def _pack_module(self, source_module_path, target_package_path, base_dir="."):
        print(f"\t\u21E8 Packing {source_module_path} into {target_package_path}\n")
        base_name, extension = os.path.splitext(target_package_path)
        shutil.make_archive(base_name, extension.lstrip('.'), source_module_path, base_dir)
        return target_package_path

    def _load_composer_file(self, full_module_name, module_path):
        path_to_composer = f"{module_path}/composer.json"
        if not os.path.isfile(path_to_composer):
            os.system(f"ls {module_path}")
            raise Exception(f"Can not build module {full_module_name}: composer.json is missing")

        with open(path_to_composer) as f:
            return json.load(f)

    def _get_magento_package_name(self, full_module_name, module_path):
        if not os.path.isfile(f"{module_path}/registration.php"):
            os.system(f"ls {module_path}")
            raise Exception(f"Can not build module {full_module_name}: registration.php is missing")

        with open(f"{module_path}/registration.php") as reg:
            l = reg.readlines()
            for line in l:
                if line.find("Aheadworks_") != -1:
                    m = re.search("Aheadworks_([^\"']+)", line)
                    return m.group(1)
                # Hyva extensions workaround
                if line.find("Hyva_") != -1:
                    m = re.search("Hyva_([^\"']+)", line)
                    return m.group(1)
        return ""

    def _prepare_for_publishing(self, module_path, magento_package_name, module_version):
        print("\t\u21E8 Cleanup filesystem...")
        self.file_manager.remove_files_and_dirs_ignore_case(
            module_path,
            ['bitbucket-pipelines.yml', 'readme.md', '.gitignore'],
            ['.git']
        )

        print("\t\u21E8 Writing license headers...")
        self.file_manager.add_license_to_php_files(module_path, magento_package_name, module_version)
        os.system(f"echo See https://aheadworks.com/end-user-license-agreement/ >> {module_path}/license.txt")

    def _upload_artifacts(self, bitbucket_workspace, bitbucket_repo_slug, filepath):
        print(f"\nUploading artifacts\n{filepath}...\n")
        bitbucket_manager = BitbucketApiManager(
            BitbucketConfig(
                bitbucket_workspace=bitbucket_workspace,
                bitbucket_repo_slug=bitbucket_repo_slug
            )
        )
        print(bitbucket_manager.upload_artifacts(filepath))



from aheadworks_core.model.parser.json import Json as JsonParser
from aheadworks_core.model.cd import Cd as cd
from aheadworks_core.api.file_manager import FileManager
from aheadworks_core.model.parser.xml import Xml as XmlParser
import subprocess
import os
import stringcase


class MagentoManager:
    """manager for magento"""

    MODULE_XML_FILE = '/etc/module.xml'
    MODULE_COMPOSER_FILE = '/composer.json'
    PLATFORM_MODULE_DEPENDENCIES = {
        'aheadworks/module-onestepcheckout' : {
            'aheadworks/module-customer-attributes' : {
                'full_module_name' : 'aheadworks/module-customer-attributes',
                'module_name' : 'module-customer-attributes',
                'platform' : 'community'
            },
            'aheadworks/module-customer-attributes-relation' : {
                'full_module_name' : 'aheadworks/module-customer-attributes-relation',
                'module_name' : 'module-customer-attributes-relation',
                'platform' : 'enterprise'
            }
        }
    }

    def __init__(self):
        self.json_parser = JsonParser()
        self.file_manager = FileManager()
        self.xml_parser = XmlParser()

    def get_module_name(self, path_to_module):
        module_xml_file = path_to_module + self.MODULE_XML_FILE
        module_composer_file = path_to_module + self.MODULE_COMPOSER_FILE
        if os.path.isfile(module_xml_file):
            module_name = self.xml_parser.get_variable_from_file('module', 'name', module_xml_file)
        else:
            full_module_name = self.json_parser.get_variable_from_file('name', module_composer_file)
            owner_module_name, module_name = full_module_name.split('/')

            module_name = module_name.replace('module-', '')
            full_module_name_arr = [
                stringcase.titlecase(owner_module_name),
                stringcase.titlecase(module_name).replace(' ', '')
            ]
            module_name = '_'.join(full_module_name_arr)
        return module_name

    def get_module_version(self, path_to_module):
        module_composer_file = path_to_module + self.MODULE_COMPOSER_FILE
        return self.json_parser.get_variable_from_file('version', module_composer_file)

    def download_modules_from_git(self, path_to_module, tmp_dir_m2_modules, git_workspace='awm2ext'):
        try:
            dir_composer = tmp_dir_m2_modules + '/composer'
            dir_app_code = tmp_dir_m2_modules + '/app/code'

            list_of_module_paths = dict({
                'dir_composer': dir_composer,
                'dir_app_code': dir_app_code,
                'module_names': list(),
                'dir_composer_with_modules': dict(),
                'dir_app_code_with_modules': dict()
            })

            module_dependencies = self.get_module_dependencies(path_to_module)

            self.file_manager.create_dir_by_path(dir_composer)
            self.file_manager.create_dir_by_path(dir_app_code)
            with cd(dir_composer):
                for full_module_name, module in module_dependencies.items():
                    module_name = module['module_name']
                    module_version = module['version']

                    proc = subprocess.Popen(
                        ['git', 'clone', 'git@bitbucket.org:' + git_workspace + '/' + module_name + '.git']
                    )
                    proc.communicate()
                    if proc.returncode != 0:
                        raise Exception('Failed download module ' + module_name)
                    with cd(module_name):
                        proc = subprocess.Popen(['git', 'checkout', module_version])
                        proc.communicate()

                    self.file_manager.remove_files_and_dirs_ignore_case(
                        module_name,
                        ['bitbucket-pipelines.yml', 'readme.md', '.gitignore'],
                        ['.git']
                    )

            for item in os.listdir(dir_composer):
                full_composer_module_path = dir_composer + '/' + item
                module_name = self.get_module_name(full_composer_module_path)
                module_path = module_name.replace('_', '/')
                full_app_code_module_path = dir_app_code + '/' + module_path
                self.file_manager.create_dir_by_path(full_app_code_module_path)
                os.system('cp -r {}/* {}/'.format(full_composer_module_path, full_app_code_module_path))
                list_of_module_paths['module_names'].append(module_name)
        except Exception as error:
            os.system('rm -rf ' + tmp_dir_m2_modules)
            raise Exception(error)

        return list_of_module_paths

    def get_platform_module_dependencies(self, module_name):
        try:
            platform_dependencies = self.PLATFORM_MODULE_DEPENDENCIES[module_name]
        except Exception:
            platform_dependencies = dict()
        return platform_dependencies

    def get_module_dependencies(self, path_to_module):
        return self.get_module_dependencies_from_composer(path_to_module + self.MODULE_COMPOSER_FILE)

    def is_suggested_module(self, path_to_module, suggested_module_name):
        composer_file = path_to_module + self.MODULE_COMPOSER_FILE
        try:
            composer_suggests = self.json_parser.get_variable_from_file('suggests', composer_file)
        except Exception:
            composer_suggests = dict()
        return suggested_module_name in composer_suggests

    def get_module_dependencies_from_composer(self, composer_file):
        list_of_modules = dict()

        core_full_module_name = self.json_parser.get_variable_from_file('name', composer_file)
        core_version = self.json_parser.get_variable_from_file('version', composer_file)
        try:
            composer_requires = self.json_parser.get_variable_from_file('require', composer_file)
        except Exception:
            composer_requires = dict()
        try:
            composer_suggests = self.json_parser.get_variable_from_file('suggests', composer_file)
        except Exception:
            composer_suggests = dict()

        core_module_name = core_full_module_name.split('/')[1]
        list_of_modules[core_full_module_name] = dict(
            {
                'module_name': core_module_name,
                'full_module_name': core_full_module_name,
                'version': core_version.strip('><=')
            }
        )

        list_of_modules.update(self._get_valid_dependency('requires', composer_requires.items(), core_full_module_name))
        list_of_modules.update(self._get_valid_dependency('suggests', composer_suggests.items(), core_full_module_name))
        list_of_modules.update(self.get_platform_module_dependencies(core_full_module_name))

        return list_of_modules

    def _get_valid_dependency(self, type, items, core_full_module_name):
        list_of_modules = dict()

        for full_module_name, version in items:
            if self._is_valid_dependency(type, full_module_name, core_full_module_name):
                module_name = full_module_name.split('/')[1]
                list_of_modules[full_module_name] = dict(
                    {'module_name': module_name, 'full_module_name': full_module_name, 'version': version.strip('><=')}
                )
        return list_of_modules.items()

    def _is_valid_dependency(self, type, full_module_name, core_full_module_name):
        exclusion_modules = dict({'aheadworks/module-faq': ['aheadworks/module-chat-bot']})

        result = False
        if type == 'requires':
            result = full_module_name.find('aheadworks') != -1
        elif type == 'suggests':
            # In general we do not build modules from suggests section
            # BUT
            # For some modules we build other modules even in case they are not directly related
            is_directly_linked = core_full_module_name in exclusion_modules and full_module_name in exclusion_modules[core_full_module_name]

            # For aheadworks/module-foo we build aheadworks/module-foo-* from suggestions as well
            is_dependent_suggestion = full_module_name.find(core_full_module_name) != -1

            # For aheadworks/module-sarp3 we build aheadworks/module-sarp2-* from suggestions as well
            is_sarp2_suggested_by_sarp3 = core_full_module_name == "aheadworks/module-sarp3" and full_module_name.startswith("aheadworks/module-sarp2-")

            result = is_directly_linked or is_dependent_suggestion or is_sarp2_suggested_by_sarp3

        return result



import os, shutil
import re
import json
import zipfile

from tempfile import mkdtemp


class VerfiedDescriptior:
    def __init__(self):
        self.val = {}

    def __get__(self, obj, objtype):
        return self.val[obj]

    def __set__(self, obj, val):
        self.val[obj] = val


class NameDescriptor(VerfiedDescriptior):

    def __set__(self, obj, val):
        if not re.match( r"^([a-z0-9_-])+/([a-z0-9_-])+$", val, re.IGNORECASE):
            raise ValueError("Name must be in <vendor>/<package name> format, \"%s\" given" % val)
        self.val[obj] = val


class VersionDescriptor(VerfiedDescriptior):

    def __set__(self, obj, val):
        # Check `version` field
        if not re.match(r"^(|v)([0-9])+\.([0-9])+\.([0-9])+(-(patch|p|dev|a|alpha|b|beta|rc)([0-9])*)?$", val):
            raise ValueError("Version must follow https://getcomposer.org/doc/04-schema.md#version"
                             "format, \"%s\" given" % val)
        self.val[obj] = val


class TypeDescriptor(VerfiedDescriptior):

    allowed = ("magento2-theme", "magento2-language", "magento2-module")

    def __set__(self, obj, val):
        if val not in self.allowed:
            raise ValueError("Type can only be one of the following:"
                             "%s; %s given." % (", ".join(self.allowed), val))
        self.val[obj] = val


class Metadata:
    """
    Composer.json wrapper with m2-extension specific options and checks
    """

    name = NameDescriptor()
    version = VersionDescriptor()
    type = TypeDescriptor()

    def __init__(self):
        pass

    def init_from_file(self, fp):

        s = fp.read()
        try:
            js = json.loads(s)
        except TypeError:
            js = json.loads(s.decode())
        self.update(js)

    def update(self, values):
        for k, v in values.items():
            setattr(self, k, v)

    def __iter__(self):
        return iter({k: getattr(self, k) for k in dir(self) if not k.startswith('__') and not callable(getattr(self, k))}.items())


class Extension:

    def __init__(self, path=None):
        self.meta = Metadata()
        self.path = path

    def __enter__(self):
        self.tempdir = mkdtemp()
        if os.path.isdir(self.path):
            self.init_from_path(self.path)
            return self

        with zipfile.ZipFile(self.path, "r") as zip_ref:
            zip_ref.extractall(self.tempdir)
            self.init_from_path(self.tempdir)
            return self

    def __exit__(self, etype, value, traceback):
        shutil.rmtree(self.tempdir)

    def init_from_path(self, path):
        self.path = path
        if not os.path.exists(os.path.join(self.path, 'registration.php')):
            raise IOError("registration.php file doesn't exist")

        with open(os.path.join(self.path, 'composer.json')) as jsfp:
            self.meta.init_from_file(jsfp)

    def init_from_zip(self, path):
        z = zipfile.ZipFile(path)
        for fn in z.namelist():
            if os.path.basename(fn) == 'composer.json':
                with z.open(fn) as fp:
                    self.meta.init_from_file(fp)
                    return
        raise IOError("No composer.json file found in {}". format(path))



import subprocess


class SshManager:

    def run_ssh_command(self, command, connection):
        #host, user, port=22
        host = connection['user'] + '@' + connection['host']

        if connection['port'] != 22:
            subprocess_params = ["ssh", "-p %d" % connection['port'], "%s" % host, command]
        else:
            subprocess_params = ["ssh", "%s" % host, command]

        output = subprocess.check_output(
            subprocess_params,
            stderr=subprocess.STDOUT,
            universal_newlines=True
        )
        output = output.splitlines()

        return output



import json
from typing import Dict
import dateutil.parser
import docker
from aheadworks_core.model.http.api_request import ApiRequest as Api
from aheadworks_bitbucket_manager.api.bitbucket_api_manager import BitbucketApiManager
import re


class DockerhubApiManager:
    """DockerHub api manager"""

    def __init__(self, docker_hub_config, bitbucket_config):
        """
        :param repo: DockerHub repository name
        """
        self.bitbucket_api_manager = BitbucketApiManager(bitbucket_config)
        self.config = docker_hub_config
        self.request = Api(config=self.config)
        self.pull_expire_date = self.config.pull_expire_date
        self.push_expire_date = self.config.push_expire_date
        self.count_image_to_save = 8

    def remove_deprecated_versions_images(self, repo):
        tags = self.get_tags(repo)
        minor_version_images = list()
        for i in tags['results']:
            match_value = re.search('[0-9]-[0-9]', i['name'])
            if match_value:
                minor_version_images.append(i)
        images = list()
        if len(minor_version_images) <= self.count_image_to_save:
            # todo refactoring, add to log
            print(f'there less than {self.count_image_to_save} images, Nothing will be removed')
            return None
        minor_version_images = sorted(minor_version_images, key=lambda x: x['name'].split('-')[-1], reverse=True)
        for i in minor_version_images[self.count_image_to_save:]:
            tag_last_pulled = i['tag_last_pulled']
            pull_date = dateutil.parser.parse(tag_last_pulled).replace(tzinfo=None) or None
            push_date = dateutil.parser.parse(i['tag_last_pushed']).replace(tzinfo=None)
            if push_date < self.push_expire_date and (pull_date < self.pull_expire_date or pull_date is None):
                self.remove_tag(i['name'], repo)
                images.append(i['name'])
                # todo refactoring, add to log
                print(f'removes {self.config.dockerhub_user}/{repo}:{i["name"]}')
        if len(images) == 0:
            # todo refactoring, add to log
            print('there no images older than 30 days. Nothing will be removed')
        return

    def get_images_by_commit(self, commit_hash: str, repo, pipe_name):
        """
        :param commit_hash:
        :param repo:
        :return:
        :raises Exception: Images for this build not exists.
        """

        build_number = self.bitbucket_api_manager.get_build_by_commit(commit_hash, pipe_name)

        tag_names = []
        tags = self.get_tags(repo)['results']
        images = list()
        for tag in tags:
            tag_names.append(tag['name'])
            if tag['name'].find(f'-{str(build_number)}') != -1:
                images.append(f'{self.config.dockerhub_user}/{repo}:{tag["name"]}')
        if len(images) == 0:
            raise Exception(
                "Images for this build not exists. DH Repo: %s, Build: %s, Commit: %s, Tags count: %s, Tags: %s"
                % (repo, build_number, commit_hash, len(tags), repr(tag_names))
            )

        return images

    def renew_images_by_tag(self, commit_hash, repo, pipe_name):
        images = self.get_images_by_commit(commit_hash, repo, pipe_name)
        self.retag_images(images, repo)
        return True

    def retag_images(self, images: list, repo):
        """
        :param images:
        :param repo:
        :return:
        """

        docker_client = self._docker_client_login()
        for i in images:
            image = docker_client.images.pull(i)
            repository, tag = i.split(':')
            new_tag = '-'.join(tag.split('-')[0:-1])
            image.tag(repository, new_tag)
            docker_client.images.push(f"{repository}:{new_tag}")
            print(f"rename {i} to {repository}:{new_tag}")
            self.remove_tag(tag, repo)

    def get_tags(self, repo):
        headers: Dict = self.config.auth_headder
        params = (('page_size', '10000'),)
        r = self.request.get(location='/v2/repositories/{}/{}/tags/?page_size=10000'.format(
                             self.config.dockerhub_user,
                             repo),
                             params=params,
                             headers=headers)
        return json.loads(r)

    def remove_tag(self, tag_name, repo):
        headers: Dict = self.config.auth_headder
        r = self.request.delete(location='/v2/repositories/{}/{}/tags/{}/'.format(
                                self.config.dockerhub_user,
                                repo,
                                tag_name),
                                headers=headers)
        return r

    def push_images(self, images):
        docker_client = self._docker_client_login()
        result = ''
        for image in images:
            for line in docker_client.images.push(image, stream=True, decode=True):
                result += str(line) + '\n'
        return result

    def _docker_client_login(self):
        docker_client = docker.from_env()
        docker_client.login(username=self.config.dockerhub_user, password=self.config.dockerhub_password)
        return docker_client



import os
import sys
import subprocess
import traceback
from aheadworks_bitbucket_manager.api.bitbucket_api_manager import BitbucketApiManager
from aheadworks_bitbucket_manager.api.dockerhub_api_manager import DockerhubApiManager
from aheadworks_bitbucket_manager.api.version_control_manager import VersionControlManager
from aheadworks_bitbucket_manager.api.file_manager import FileManager
from aheadworks_bitbucket_manager.api.db_manager import DbManager
from aheadworks_bitbucket_manager.model.parser.php import Php as PhpParser
from aheadworks_bitbucket_manager.model.data.dockerhub import DockerHubConfig
from aheadworks_bitbucket_manager.model.data.bitbucket import BitbucketConfig
from aheadworks_bitbucket_manager.model.ssh_manager import SshManager


class Console:
    """
    this application needed next env variables
    BITBUCKET_KEY
    BITBUCKET_SECRET
    BITBUCKET_REPO_SLUG
    BITBUCKET_WORKSPACE
    DOCKERHUB_LOGIN
    DOCKERHUB_PASSWORD
    DOCKERHUB_REPO
    """

    def __init__(self):
        self.available_pipe_name = [
            'build',
            'release/*',
            'build-and-deploy-to-staging-sandbox',
            'build-and-deploy-to-staging-box',
            'deploy-to-staging-jetrails',
            'build-and-deploy-to-staging-jetrails'
        ]
        docker_hub_config = DockerHubConfig(
            dockerhub_user=os.getenv('DOCKERHUB_LOGIN'),
            dockerhub_password=os.getenv('DOCKERHUB_PASSWORD')
        )
        bitbucket_config = BitbucketConfig(
            bitbucket_key=os.getenv('BITBUCKET_KEY'),
            bitbucket_secret=os.getenv('BITBUCKET_SECRET'),
            bitbucket_workspace=os.getenv('BITBUCKET_WORKSPACE'),
            bitbucket_repo_slug=os.getenv('BITBUCKET_REPO_SLUG')
        )

        self.bitbucket_api_manager = BitbucketApiManager(bitbucket_config)
        self.docker_hub_api_manager = DockerhubApiManager(docker_hub_config, bitbucket_config)
        self.file_manager = FileManager()
        self.version_control_manager = VersionControlManager()
        self.db_manager = DbManager()
        self.php_parser = PhpParser()
        self.ssh_manager = SshManager()

    # Docker Hub
    def remove_deprecated_versions_images(self, repo=None):
        """
        :param repo:
        :return:
        """

        repo = repo or os.getenv('DOCKERHUB_REPO')
        self.docker_hub_api_manager.remove_deprecated_versions_images(repo)

    def renew_images_by_tag(self, commit_hash, repo=None, pipe_name=None):
        """
        :param commit_hash:
        :param repo:
        :param pipe_name:
        :return:
        """
        try:
            if pipe_name is None:
                pipe_name = self.available_pipe_name

            exit_code = 0
            repo = repo or os.getenv('DOCKERHUB_REPO')
            self.docker_hub_api_manager.renew_images_by_tag(commit_hash, repo, pipe_name)
        except Exception as error:
            print('Error: ' + repr(error))
            for k, v in os.environ.items():
                print(f'{k}={v}')
            exit_code = 1

        exit(exit_code)

    def push_images(self, images):
        print(self.docker_hub_api_manager.push_images(images))

    # Version Control
    def run_common_app(self, check_containers, up_cmd, host=None, user=None, port=22):
        try:
            connection = self._get_default_ssh_cred(host, user, port)
            print(self.version_control_manager.run_common_app(check_containers, up_cmd, connection))
            exit_code = 0
        except subprocess.CalledProcessError as error:
            print('Error: ' + error.output)
            exit_code = 1
        except Exception as error:
            print('Error: ' + repr(error))
            exit_code = 1

        exit(exit_code)

    def run_app(
            self,
            path_to_versions,
            path_to_project,
            up_cmd,
            down_exclude_current,
            down_cmd,
            host=None,
            user=None
    ):
        try:
            connection = self._get_default_ssh_cred(host, user)
            print(self.version_control_manager.run_app(
                path_to_versions,
                path_to_project,
                up_cmd,
                down_exclude_current,
                down_cmd,
                connection
            ))
            exit_code = 0
        except subprocess.CalledProcessError as error:
            print('Error: ' + error.output)
            traceback.print_exc()
            exit_code = 1
        except Exception as error:
            print('Error: ' + repr(error))
            traceback.print_exc()
            exit_code = 1

        exit(exit_code)

    def run_app_from_json_config(
            self,
            path_to_versions,
            path_to_project,
            path_to_json_config,
            down_exclude_current,
            host=None,
            user=None
    ):
        try:
            connection = self._get_default_ssh_cred(host, user)
            print(self.version_control_manager.run_app_from_json_config(
                path_to_versions,
                path_to_project,
                path_to_json_config,
                down_exclude_current,
                connection
            ))
            exit_code = 0
        except subprocess.CalledProcessError as error:
            print('Error: ' + error.output)
            traceback.print_exc()
            exit_code = 1
        except Exception as error:
            print('Error: ' + repr(error))
            traceback.print_exc()
            exit_code = 1

        exit(exit_code)

    def modify_env(self, path, template):
        """Create an .env file from a template file

        Get .env file template, parse %VARS% substitute :-default value
        current environment and store the result to another location.
        Samples: VAR=%ANOTHER_VAR%, VAR=%$ANOTHER_VAR:-default%
        """

        self.version_control_manager.modify_env(path, template)

    def generate_env(self, source_file_path, target_file_path, verbose=False):
        """Create an .env file from a sample .env file

        Get a sample .env file, override values with the values from the
        current environment and store the result to another location
        """

        self.version_control_manager.generate_env(source_file_path, target_file_path, verbose)

    # Deprecated
    def get_version_uid(self, version):
        print(self.version_control_manager.get_version_uid(version))

    def get_traefik_version_query(self, path_to_versions):
        print(self.version_control_manager.get_traefik_version_query(path_to_versions))

    # --priority_type low or --priority_type height
    def get_free_priority(self, label_name, priority_type, host=None, user=None):
        connection = self._get_default_ssh_cred(host, user)
        try:
            print(self.version_control_manager.get_free_priority(label_name, priority_type, connection))
        except Exception:
            exit(1)

    def get_free_port(self, host=None, user=None):
        connection = self._get_default_ssh_cred(host, user)
        try:
            print(self.version_control_manager.get_free_port(connection))
        except Exception:
            exit(1)

    def get_variable_from_php_file(self, var_name, file):
        print(self.php_parser.get_variable_from_file(var_name, file))

    # Db manager
    def create_dump(self, path_to_sh, host=None, user=None):
        connection = self._get_default_ssh_cred(host, user)
        print(self.db_manager.create_dump(path_to_sh, connection))

    # File Manager
    def add_info_to_file_header(self, directories: list, file_extensions: list, info_string: str):
        for _ in self.file_manager.add_info_to_file_header(directories, file_extensions, info_string):
            print(_)

    # Bitbucket Api
    def get_build_by_commit(self, commit_hash: str, pipe_name=None, depth=1):
        try:
            if pipe_name is None:
                pipe_name = self.available_pipe_name

            build_number = self.bitbucket_api_manager.get_build_by_commit(commit_hash, pipe_name, depth)
            if build_number is None:
                raise Exception('Build not found.')
            else:
                print(build_number)
        except Exception as error:
            print('Error: ' + repr(error), file=sys.stderr)
            exit(1)

    # Ssh manager
    def run_ssh_command(self, cmd, host=None, user=None):
        try:
            connection = self._get_default_ssh_cred(host, user)
            run_result = self.ssh_manager.run_ssh_command(cmd, connection)
            run_result = type(run_result) == list and '\n'.join(run_result)
            print('run_result:\n' + run_result)
            exit_code = 0
        except subprocess.CalledProcessError as error:
            print('Error: ' + error.output)
            exit_code = 1
        except Exception as error:
            print('Error: ' + repr(error))
            exit_code = 1

        exit(exit_code)

    def _get_default_ssh_cred(self, host, user, port=22):
        user = user or os.getenv('HOST_USER') or os.getenv('SSH_USER') or 'root'
        host = host or os.getenv('HOST') or os.getenv('SSH_HOST') or ''
        port = int(os.getenv('SSH_PORT')) or 22
        connection = {
            'host': host,
            'user': user,
            'port': port
        }
        return connection

    def internal_composer_refresh(self):
        try:
            internal_composer_repo = os.getenv('COMPOSER_DEV_REPO')
            target_repo = os.getenv('BITBUCKET_GIT_SSH_ORIGIN')
            data = {
                "target": {
                    "type": "pipeline_ref_target",
                    "ref_type": "branch",
                    "ref_name": "master",
                    "selector": {
                        "type": "custom",
                        "pattern": "rebuild-selected-packages"
                    }
                },
                "variables": [
                    {
                        "key": "PACKAGE_URL",
                        "value": target_repo
                    }
                ]
            }
            print(self.bitbucket_api_manager.run_pipeline(internal_composer_repo, data))
        except Exception as error:
            print('Error: ' + repr(error), file=sys.stderr)
            exit(1)
